//	FileLoader.cpp - Class to load and save the SlotValues struct.
//	----------------------------------------------------------------------------
//	This file is part of Mouse To OSC, a simple OSC-outputting program.
//	Copyright (c) 2006 Niall Moody.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//	----------------------------------------------------------------------------

#include <juce.h>
#include "FileLoader.h"
#include "window.h"

//------------------------------------------------------------------------------
FileLoader::FileLoader(const String& path):
filePath(path)
{
	
}

//------------------------------------------------------------------------------
FileLoader::~FileLoader()
{
	
}

//------------------------------------------------------------------------------
void FileLoader::loadFile(SlotValues *slots,
				  		  String& ipAddress,
				  		  String& port,
				  		  int& midiInput)
{
	int padNumber;
	int tSNumber;
	int bSNumber;
	XmlElement *tempEl;
	XmlElement *tempEl2;
	int slotNumber = 0;
	File tempFile(filePath);
	XmlDocument doc(tempFile);
	XmlElement *root = doc.getDocumentElement();

	if(root)
	{
		if(root->hasTagName(T("MouseToOSC")))
		{
			//Read in ipAddress, port and midiInput first.
			ipAddress = root->getStringAttribute(T("ipAddress"));
			port = root->getStringAttribute(T("port"));
			midiInput = root->getIntAttribute(T("midiInput"));

			//Now read in all the slider values etc.
			for(tempEl=root->getFirstChildElement();
				tempEl;
				tempEl=tempEl->getNextElement())
			{
				if(tempEl->hasTagName(T("Slot")))
				{
					padNumber = 0;
					tSNumber = 0;
					bSNumber = 0;
					for(tempEl2=tempEl->getFirstChildElement();
						tempEl2;
						tempEl2=tempEl2->getNextElement())
					{
						if(tempEl2->hasTagName(T("PadControl")))
						{
							slots[slotNumber].padValues[padNumber].xAddress = tempEl2->getStringAttribute(T("xAddress"));
							slots[slotNumber].padValues[padNumber].yAddress = tempEl2->getStringAttribute(T("yAddress"));
							slots[slotNumber].padValues[padNumber].xMax = tempEl2->getStringAttribute(T("xMax"));
							slots[slotNumber].padValues[padNumber].yMax = tempEl2->getStringAttribute(T("yMax"));
							slots[slotNumber].padValues[padNumber].xMin = tempEl2->getStringAttribute(T("xMin"));
							slots[slotNumber].padValues[padNumber].yMin = tempEl2->getStringAttribute(T("yMin"));
							slots[slotNumber].padValues[padNumber].xVal = tempEl2->getDoubleAttribute(T("xVal"));
							slots[slotNumber].padValues[padNumber].yVal = tempEl2->getDoubleAttribute(T("yVal"));
							slots[slotNumber].padValues[padNumber].xCc = tempEl2->getStringAttribute(T("xCc"));
							slots[slotNumber].padValues[padNumber].yCc = tempEl2->getStringAttribute(T("yCc"));

							++padNumber;
						}
						else if(tempEl2->hasTagName(T("TopSliderControl")))
						{
							slots[slotNumber].topSliderValues[tSNumber].address = tempEl2->getStringAttribute(T("address"));
							slots[slotNumber].topSliderValues[tSNumber].max = tempEl2->getStringAttribute(T("max"));
							slots[slotNumber].topSliderValues[tSNumber].min = tempEl2->getStringAttribute(T("min"));
							slots[slotNumber].topSliderValues[tSNumber].val = tempEl2->getDoubleAttribute(T("val"));
							slots[slotNumber].topSliderValues[tSNumber].cc = tempEl2->getStringAttribute(T("cc"));

							++tSNumber;
						}
						else if(tempEl2->hasTagName(T("BottomSliderControl")))
						{
							slots[slotNumber].bottomSliderValues[bSNumber].address = tempEl2->getStringAttribute(T("address"));
							slots[slotNumber].bottomSliderValues[bSNumber].max = tempEl2->getStringAttribute(T("max"));
							slots[slotNumber].bottomSliderValues[bSNumber].min = tempEl2->getStringAttribute(T("min"));
							slots[slotNumber].bottomSliderValues[bSNumber].val = tempEl2->getDoubleAttribute(T("val"));
							slots[slotNumber].bottomSliderValues[bSNumber].cc = tempEl2->getStringAttribute(T("cc"));

							++bSNumber;
						}
					}
					++slotNumber;
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
void FileLoader::saveFile(SlotValues *slots,
						  const String& ipAddress,
						  const String& port,
						  int midiInput)
{
	int i, j;
	XmlElement rootElement(T("MouseToOSC"));
	XmlElement *slotElement;
	XmlElement *controlElement;

	//First write the ipAddress, port, and midiInput attributes.
	rootElement.setAttribute(T("ipAddress"), ipAddress);
	rootElement.setAttribute(T("port"), port);
	rootElement.setAttribute(T("midiInput"), midiInput);

	for(i=0;i<8;++i)
	{
		slotElement = new XmlElement(T("Slot"));

		//Pads.
		for(j=0;j<3;++j)
		{
			controlElement = new XmlElement(T("PadControl"));

			controlElement->setAttribute(T("xAddress"),
									     slots[i].padValues[j].xAddress);
			controlElement->setAttribute(T("yAddress"),
									     slots[i].padValues[j].yAddress);
			controlElement->setAttribute(T("xMax"),
									     slots[i].padValues[j].xMax);
			controlElement->setAttribute(T("yMax"),
									     slots[i].padValues[j].yMax);
			controlElement->setAttribute(T("xMin"),
									     slots[i].padValues[j].xMin);
			controlElement->setAttribute(T("yMin"),
									     slots[i].padValues[j].yMin);
			controlElement->setAttribute(T("xVal"),
									     slots[i].padValues[j].xVal);
			controlElement->setAttribute(T("yVal"),
									     slots[i].padValues[j].yVal);
			controlElement->setAttribute(T("xCc"),
									     slots[i].padValues[j].xCc);
			controlElement->setAttribute(T("yCc"),
									     slots[i].padValues[j].yCc);

			slotElement->addChildElement(controlElement);
		}

		//Top sliders.
		for(j=0;j<3;++j)
		{
			controlElement = new XmlElement(T("TopSliderControl"));

			controlElement->setAttribute(T("address"),
									  	 slots[i].topSliderValues[j].address);
			controlElement->setAttribute(T("max"),
									  	 slots[i].topSliderValues[j].max);
			controlElement->setAttribute(T("min"),
									  	 slots[i].topSliderValues[j].min);
			controlElement->setAttribute(T("val"),
									  	 slots[i].topSliderValues[j].val);
			controlElement->setAttribute(T("cc"),
									  	 slots[i].topSliderValues[j].cc);

			slotElement->addChildElement(controlElement);
		}

		//Bottom sliders.
		for(j=0;j<3;++j)
		{
			controlElement = new XmlElement(T("BottomSliderControl"));

			controlElement->setAttribute(T("address"),
									  	 slots[i].bottomSliderValues[j].address);
			controlElement->setAttribute(T("max"),
									  	 slots[i].bottomSliderValues[j].max);
			controlElement->setAttribute(T("min"),
									 	 slots[i].bottomSliderValues[j].min);
			controlElement->setAttribute(T("val"),
									  	 slots[i].bottomSliderValues[j].val);
			controlElement->setAttribute(T("cc"),
									  	 slots[i].bottomSliderValues[j].cc);

			slotElement->addChildElement(controlElement);
		}

		rootElement.addChildElement(slotElement);
	}

	File tempFile(filePath);
	rootElement.writeToFile(tempFile, T(""));
}
