//	PhilChooser.cpp - File chooser using the Phil Component.
//	--------------------------------------------------------------------------
//	This file is part of 'Phil', a very simple file manager/browser for the
//	JUCE library.
//	Copyright (C) 2005-2006  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------

#include <juce.h>
#include "PhilChooser.h"
#include "PhilComponent.h"
#include "StandardPhilToolbar.h"
#include "PhilChooserStatusbar.h"

//----------------------------------------------------------------------------
PhilChooser::PhilChooser(const String& dialogBoxTitle,
						 const File& initialFileOrDirectory,
						 const String& filePatternsAllowed):
DialogWindow(dialogBoxTitle, Colours::white, false),
toolbar(0),
statusbar(0),
sidebar(0),
returnVal(false),
directorySelect(false),
fileOpen(false)
{
	setTitleBarTextCentred(false);

	philComponent = new Phil();
	addKeyListener(philComponent->getKeyMappings());

	setupExtensions(filePatternsAllowed);

	philComponent->setToolbarComponent(toolbar = new StandardPhilToolbar(philComponent));
	philComponent->setStatusbarComponent(statusbar = new PhilChooserStatusbar(philComponent));
	philComponent->setStatusbarHeight(50);
	statusbar->addChangeListener(this);
	if(initialFileOrDirectory != File::nonexistent)
		philComponent->setCurrentDirectory(initialFileOrDirectory);
	//JUCE only allows for single files to be selected at a time - we keep to
	//that principal for compatibility.
	philComponent->setMultipleFilesSelected();
	philComponent->setAllowExternalPrograms(false);
	philComponent->setAllowDragAndDrop(false);
	setContentComponent(philComponent);

	//setOpaque(true);
}

//----------------------------------------------------------------------------
PhilChooser::~PhilChooser()
{
	//deleteAllChildren();
}

//----------------------------------------------------------------------------
bool PhilChooser::browseForFileToOpen(FilePreviewComponent *extraInfoComponent)
{
	if(extraInfoComponent)
	{
		sidebar = extraInfoComponent;
		philComponent->setSidebarComponent(sidebar);
	}

	directorySelect = false;
	fileOpen = true;

	centreWithSize(410, 300);
	setAlwaysOnTop(true);

	if(runModalLoop() > 0)
	{
		if(sidebar)
		{
			philComponent->removeChildComponent(sidebar);
			delete sidebar;
			sidebar = 0;
		}
		return true;
	}
	else
	{
		if(sidebar)
		{
			philComponent->removeChildComponent(sidebar);
			delete sidebar;
			sidebar = 0;
		}
		return false;
	}
}

//----------------------------------------------------------------------------
bool PhilChooser::browseForFileToSave()
{
	int i = 0;
	bool extensionFound = false;
	const StringArray &tempExtensions = philComponent->getVisibleExtensions();

	directorySelect = false;
	fileOpen = false;

	centreWithSize(410, 300);
	setAlwaysOnTop(true);

	if(runModalLoop() > 0)
	{
		if(tempExtensions.size() > 0)
		{
			while((!extensionFound)&&(i < tempExtensions.size()))
			{
				if(returnFile.getFullPathName().endsWith(tempExtensions[i]))
					extensionFound = true;
				++i;
			}
			if(!extensionFound)
			{
				String tempstr = returnFile.getFullPathName();
				tempstr << T(".") << tempExtensions[0];
				returnFile = File(tempstr);
			}
		}
		return true;
	}
	else
		return false;
}

//----------------------------------------------------------------------------
bool PhilChooser::browseForDirectory()
{
	directorySelect = true;
	fileOpen = false;

	centreWithSize(410, 300);
	setAlwaysOnTop(true);

	if(runModalLoop() > 0)
		return true;
	else
		return false;
}

//----------------------------------------------------------------------------
const File PhilChooser::getResult() const
{
	return returnFile;
}

//----------------------------------------------------------------------------
void PhilChooser::closeButtonPressed()
{
	//setVisible(false);
	exitModalState(0);
}

//----------------------------------------------------------------------------
void PhilChooser::changeListenerCallback(void *objectThatHasChanged)
{
	if(objectThatHasChanged == philComponent)
	{
		if(sidebar)
		{
			const StringArray& selected = philComponent->getCurrentlyHighlightedFiles();
			FilePreviewComponent *temp = static_cast<FilePreviewComponent *>(objectThatHasChanged);

			if(selected.size() > 0)
			{
				File tempFile(selected[0]);
				temp->selectedFileChanged(tempFile);
			}
			else
				temp->selectedFileChanged(File::nonexistent);
		}
	}
	else if(objectThatHasChanged == statusbar)
	{
		const String& tempstr = statusbar->getFile();

		if(tempstr == String::empty)
			closeButtonPressed();
		else
		{
			String tempstr2 = philComponent->getCurrentDirectory().getFullPathName();
			tempstr2 << File::separator << tempstr;
			File tempFile(tempstr2);

			//If we just want to open a directory.
			if((!directorySelect) && (tempFile.isDirectory()))
			{
				philComponent->setCurrentDirectory(tempFile);
				return;
			}
			//If we want to return a directory.
			else if(directorySelect)
				returnFile = tempFile;
			//If we want to return an existing file.
			else if(fileOpen)
			{
				if(tempFile.existsAsFile())
					returnFile = tempFile;
				else
				{
					closeButtonPressed();
					//Is this actually necessary?
					return;
				}
			}
			//If we don't mind whether the file exists or not.
			else
				returnFile = tempFile;

			exitModalState(1);
		}
	}
}

//----------------------------------------------------------------------------
void PhilChooser::setColours(const Colour& background,
							 const Colour& boundary,
							 const Colour& text,
							 const Colour& highlight)
{
	setBackgroundColour(background);
	philComponent->setColours(background, boundary, text, highlight);
}

//----------------------------------------------------------------------------
void PhilChooser::paint(Graphics &g)
{
	g.setColour(philComponent->getBackgroundColour());
	g.fillAll();

	/*g.setColour(phil.getTextColour());
	g.setFont(16, Font::bold);

	g.drawSingleLineText(T("Phil Browser"), 6, 17);*/

	DialogWindow::paint(g);
}

//----------------------------------------------------------------------------
void PhilChooser::setupExtensions(const String& str)
{
	String origstr = str;
	String tempstr1;
	StringArray allowedExtensions;

	if(str != String::empty)
	{
		//To make it easier to parse the string (only have to look for ';').
		origstr = origstr.replaceCharacter(',', ';');

		origstr << ';'; //So the String ends with ';'.

		tempstr1 = origstr.upToFirstOccurrenceOf(T(";"), false, true);
		if(tempstr1 == T("*"))
			allowedExtensions.add(tempstr1);
		else
			allowedExtensions.add(tempstr1.substring(2, tempstr1.length()));

		if(tempstr1.length() != (origstr.length()-1))
		{
			origstr = origstr.fromFirstOccurrenceOf(T(";"), false, true);
			tempstr1 = origstr.upToFirstOccurrenceOf(T(";"), false, true);
			while(tempstr1 != origstr)
			{
				if(tempstr1 == T("*"))
					allowedExtensions.add(tempstr1);
				else
					allowedExtensions.add(tempstr1.substring(2,
															  tempstr1.length()));

				origstr = origstr.fromFirstOccurrenceOf(T(";"), false, true);
				tempstr1 = origstr.upToFirstOccurrenceOf(T(";"), false, true);
			}
		}

		philComponent->maskExtensions(allowedExtensions);
	}
}
